=== modified file '.bzrignore'
--- .bzrignore	2006-11-10 01:55:55 +0000
+++ .bzrignore	2007-04-12 20:27:42 +0000
@@ -33,3 +33,4 @@
 ./.perf_history
 ./html_docs
 ./pretty_docs
+./api

=== modified file 'Makefile'
--- Makefile	2007-04-11 08:09:06 +0000
+++ Makefile	2007-04-12 20:27:42 +0000
@@ -1,3 +1,25 @@
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# A relatively simple Makefile to assist in building parts of bzr. Mostly for
+# building documentation, etc.
+
+.PHONY: all clean pyflakes api-docs
+
+
 all:
 
 check:
@@ -23,7 +45,13 @@
 	-find . -name "*.pyc" -o -name "*.pyo" | xargs rm -f
 	rm -rf test????.tmp
 
-.PHONY: all
+docfiles = bzr bzrlib
+api-docs:
+	mkdir -p api/html
+	PYTHONPATH=$(PWD) python tools/bzr_epydoc --html -o api/html --docformat 'restructuredtext en' $(docfiles)
+
+check-api-docs:
+	PYTHONPATH=$(PWD) python tools/bzr_epydoc --check --docformat 'restructuredtext en' $(docfiles)
 
 
 # build emacs cross-reference

=== modified file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	2006-10-16 01:25:46 +0000
+++ bzrlib/lazy_import.py	2007-04-12 20:27:42 +0000
@@ -48,7 +48,12 @@
     needed.
     """
 
-    __slots__ = ('_scope', '_factory', '_name')
+    __slots__ = ('_scope', '_factory', '_name', '_real_obj')
+
+    # Setting this to True will allow you to do x = y, and still access members
+    # from both variables. This should not normally be enabled, but is useful
+    # when building documentation.
+    _should_proxy = False
 
     def __init__(self, scope, factory, name):
         """Create a temporary object in the specified scope.
@@ -62,6 +67,7 @@
         self._scope = scope
         self._factory = factory
         self._name = name
+        self._real_obj = None
         scope[name] = self
 
     def _replace(self):
@@ -81,6 +87,8 @@
                           " to another variable?",
                 extra=e)
         obj = factory(self, scope, name)
+        if ScopeReplacer._should_proxy:
+            self._real_obj = obj
         scope[name] = obj
         return obj
 
@@ -92,10 +100,12 @@
         # del self._name
 
     def __getattribute__(self, attr):
-        _replace = object.__getattribute__(self, '_replace')
-        obj = _replace()
-        _cleanup = object.__getattribute__(self, '_cleanup')
-        _cleanup()
+        obj = object.__getattribute__(self, '_real_obj')
+        if obj is None:
+            _replace = object.__getattribute__(self, '_replace')
+            obj = _replace()
+            _cleanup = object.__getattribute__(self, '_cleanup')
+            _cleanup()
         return getattr(obj, attr)
 
     def __call__(self, *args, **kwargs):

=== modified file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_lazy_import.py	2007-04-12 20:27:42 +0000
@@ -96,6 +96,15 @@
     get collisions.
     """
 
+    def setUp(self):
+        TestCase.setUp(self)
+        # These tests assume we will not be proxying, so make sure proxying is
+        # disabled.
+        orig_proxy = lazy_import.ScopeReplacer._should_proxy
+        def restore():
+            lazy_import.ScopeReplacer._should_proxy = orig_proxy
+        lazy_import.ScopeReplacer._should_proxy = False
+
     def test_object(self):
         """ScopeReplacer can create an instance in local scope.
         
@@ -326,16 +335,74 @@
         self.assertRaises(errors.IllegalUseOfScopeReplacer,
                           getattr, test_obj3, 'foo')
         
-        # However, the 
-        self.assertEqual([('__getattribute__', 'foo'),
-                          '_replace',
-                          'factory',
-                          'init',
-                          ('foo', 1),
-                          ('foo', 2),
-                          ('foo', 3),
-                          ('__getattribute__', 'foo'),
-                          '_replace',
+        self.assertEqual([('__getattribute__', 'foo'),
+                          '_replace',
+                          'factory',
+                          'init',
+                          ('foo', 1),
+                          ('foo', 2),
+                          ('foo', 3),
+                          ('__getattribute__', 'foo'),
+                          '_replace',
+                         ], actions)
+
+    def test_enable_proxying(self):
+        """Test that we can allow ScopeReplacer to proxy."""
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            return TestClass()
+
+        try:
+            test_obj4
+        except NameError:
+            # test_obj4 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj4 was not supposed to exist yet')
+
+        lazy_import.ScopeReplacer._should_proxy = True
+        InstrumentedReplacer(scope=globals(), name='test_obj4',
+                             factory=factory)
+
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj4, '__class__'))
+        test_obj5 = test_obj4
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj4, '__class__'))
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj5, '__class__'))
+
+        # The first use of the alternate variable causes test_obj2 to
+        # be replaced.
+        self.assertEqual('foo', test_obj4.foo(1))
+        self.assertEqual(TestClass,
+                         object.__getattribute__(test_obj4, '__class__'))
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj5, '__class__'))
+        # We should be able to access test_obj4 attributes normally
+        self.assertEqual('foo', test_obj4.foo(2))
+        # because we enabled proxying, test_obj5 can access its members as well
+        self.assertEqual('foo', test_obj5.foo(3))
+        self.assertEqual('foo', test_obj5.foo(4))
+
+        # However, it cannot be replaced by the ScopeReplacer
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj5, '__class__'))
+
+        self.assertEqual([('__getattribute__', 'foo'),
+                          '_replace',
+                          'factory',
+                          'init',
+                          ('foo', 1),
+                          ('foo', 2),
+                          ('__getattribute__', 'foo'),
+                          ('foo', 3),
+                          ('__getattribute__', 'foo'),
+                          ('foo', 4),
                          ], actions)
 
 

=== added file 'tools/bzr_epydoc'
--- tools/bzr_epydoc	1970-01-01 00:00:00 +0000
+++ tools/bzr_epydoc	2007-04-12 20:27:42 +0000
@@ -0,0 +1,14 @@
+#!/usr/bin/env python
+#
+# Call the command line interface for Epydoc.
+#
+
+"""Wrapper around Epydoc that just makes it understand bzr's lazy imports."""
+
+# This will enable the lazy_import compatibility code
+import bzr_epydoc_uid
+
+from epydoc.cli import cli
+cli()
+
+

=== added file 'tools/bzr_epydoc_uid.py'
--- tools/bzr_epydoc_uid.py	1970-01-01 00:00:00 +0000
+++ tools/bzr_epydoc_uid.py	2007-04-12 20:27:42 +0000
@@ -0,0 +1,43 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Monkey patch to make epydoc work with bzrlib's lazy imports."""
+
+import epydoc.uid
+
+import bzrlib.lazy_import
+
+
+_ObjectUID = epydoc.uid.ObjectUID
+_ScopeReplacer = bzrlib.lazy_import.ScopeReplacer
+
+
+class ObjectUID(_ObjectUID):
+
+    def __init__(self, obj):
+        if isinstance(obj, _ScopeReplacer):
+            # The isinstance will trigger a replacement if it is a real
+            # _BzrScopeReplacer, but the local object won't know about it, so
+            # replace it locally.
+            obj = object.__getattribute__(obj, '_real_obj')
+        _ObjectUID.__init__(self, obj)
+
+
+epydoc.uid.ObjectUID = ObjectUID
+
+
+_ScopeReplacer._should_proxy = True
+

